@@ -7,10 +7,12 @@ use std::os::unix::fs::PermissionsExt;
 
 use crate::filetype::*;
 use crate::seek_forward::SeekForward;
-use crate::{align_to_4_bytes, SeenFiles};
+use crate::SeenFiles;
 
+const CPIO_ALIGNMENT: u32 = 4;
 const CPIO_HEADER_LENGTH: u32 = 110;
 const CPIO_MAGIC_NUMBER: [u8; 6] = *b"070701";
+const PATH_MAX: usize = 4096;
 
 #[derive(Debug, PartialEq)]
 pub struct Header {
@@ -131,6 +133,10 @@ impl Header {
         ]
     }
 
+    fn padding_needed_for_file_content(&self) -> u32 {
+        padding_needed_for(self.filesize.into(), CPIO_ALIGNMENT)
+    }
+
     pub fn permission(&self) -> Permissions {
         PermissionsExt::from_mode(self.mode & MODE_PERMISSION_MASK)
     }
@@ -165,18 +171,8 @@ impl Header {
         })
     }
 
-    pub fn read_only_filesize_and_filename<R: Read>(archive: &mut R) -> Result<(u32, String)> {
-        let mut header = [0; CPIO_HEADER_LENGTH as usize];
-        archive.read_exact(&mut header)?;
-        check_begins_with_cpio_magic_header(&header)?;
-        let filesize = hex_str_to_u32(&header[54..62])?;
-        let namesize = hex_str_to_u32(&header[94..102])?;
-        let filename = read_filename(archive, namesize)?;
-        Ok((filesize, filename))
-    }
-
     pub fn read_symlink_target<R: Read>(&self, archive: &mut R) -> Result<String> {
-        let align = align_to_4_bytes(self.filesize);
+        let align = self.padding_needed_for_file_content();
         let mut target_bytes = vec![0u8; (self.filesize + align).try_into().unwrap()];
         archive.read_exact(&mut target_bytes)?;
         target_bytes.truncate(self.filesize.try_into().unwrap());
@@ -186,12 +182,15 @@ impl Header {
     }
 
     pub fn skip_file_content<R: SeekForward>(&self, archive: &mut R) -> Result<()> {
-        if self.filesize == 0 {
+        skip_file_content(archive, self.filesize)
+    }
+
+    pub fn skip_file_content_padding<R: SeekForward>(&self, archive: &mut R) -> Result<()> {
+        let skip = self.padding_needed_for_file_content();
+        if skip == 0 {
             return Ok(());
         };
-        let skip = self.filesize + align_to_4_bytes(self.filesize);
-        archive.seek_forward(skip.into())?;
-        Ok(())
+        archive.seek_forward(skip.into())
     }
 
     pub fn try_get_hard_link_target<'a>(&self, seen_files: &'a SeenFiles) -> Option<&'a String> {
@@ -202,18 +201,41 @@ impl Header {
     }
 
     pub fn write<W: Write>(&self, file: &mut W) -> Result<u64> {
+        self.write_with_alignment(file, None, 0)
+    }
+
+    pub fn write_with_alignment<W: Write>(
+        &self,
+        file: &mut W,
+        alignment: Option<u32>,
+        written: u64,
+    ) -> Result<u64> {
         // The filename needs to be terminated with \0.
-        let filename_len: u32 = match (self.filename.len() + 1).try_into() {
-            Ok(l) => l,
-            Err(_) => {
-                return Err(Error::new(
-                    ErrorKind::InvalidData,
-                    format!("Path '{}' exceeds filename length limit", self.filename),
-                ))
+        let mut filename_len = self.filename.len().checked_add(1).unwrap();
+        if filename_len > PATH_MAX {
+            return Err(Error::new(
+                ErrorKind::InvalidData,
+                format!("Path '{}' exceeds filename length limit", self.filename),
+            ));
+        }
+        let offset = u64::from(CPIO_HEADER_LENGTH) + u64::try_from(filename_len).unwrap();
+        let mut padding_len;
+        if alignment.is_some_and(|alignment| self.filesize >= alignment) {
+            padding_len = padding_needed_for(written + offset, alignment.unwrap());
+            let filename_plus_alignment = filename_len
+                .checked_add(padding_len.try_into().unwrap())
+                .unwrap();
+            if filename_plus_alignment > PATH_MAX {
+                // Required padding exceeds namesize maximum. Use normal padding.
+                padding_len = padding_needed_for(offset, CPIO_ALIGNMENT);
+            } else {
+                filename_len = filename_plus_alignment;
             }
-        };
-        let padding_len = align_to_4_bytes(CPIO_HEADER_LENGTH + filename_len);
-        let padding = [0u8; 5];
+        } else {
+            padding_len = padding_needed_for(offset, CPIO_ALIGNMENT);
+        }
+        let padding = vec![0u8; (padding_len + 1).try_into().unwrap()];
+        let filename_len: u32 = filename_len.try_into().unwrap();
         write!(
             file,
             "{}{:08X}{:08X}{:08X}{:08X}{:08X}{:08X}{:08X}{:08X}{:08X}{:08X}{:08X}{:08X}00000000{}{}",
@@ -221,9 +243,19 @@ impl Header {
             self.mode, self.uid, self.gid, self.nlink, self.mtime, self.filesize,
             self.major, self.minor, self.rmajor, self.rminor,
             filename_len, self.filename,
-            std::str::from_utf8(&padding[0..(padding_len as usize)+1]).unwrap(),
+            std::str::from_utf8(&padding).unwrap(),
         )?;
-        Ok((CPIO_HEADER_LENGTH + filename_len + padding_len).into())
+        Ok(offset + u64::from(padding_len))
+    }
+
+    pub fn write_file_data_padding<W: Write>(&self, file: &mut W) -> Result<u64> {
+        let padding_len = self.padding_needed_for_file_content();
+        if padding_len == 0 {
+            return Ok(0);
+        }
+        let padding = vec![0u8; padding_len.try_into().unwrap()];
+        file.write_all(&padding)?;
+        Ok(padding_len.into())
     }
 }
 
@@ -260,8 +292,19 @@ fn hex_str_to_u32(bytes: &[u8]) -> Result<u32> {
     }
 }
 
+/// Returns the amount of padding needed after `offset` to ensure that the
+/// following address will be aligned to `alignment`.
+fn padding_needed_for(offset: u64, alignment: u32) -> u32 {
+    // The rem operation is expected smaller than the right-hand side
+    let misalignment = (offset % u64::from(alignment)) as u32;
+    if misalignment == 0 {
+        return 0;
+    }
+    alignment - misalignment
+}
+
 fn read_filename<R: Read>(archive: &mut R, namesize: u32) -> Result<String> {
-    let header_align = align_to_4_bytes(CPIO_HEADER_LENGTH + namesize);
+    let header_align = padding_needed_for((CPIO_HEADER_LENGTH + namesize).into(), CPIO_ALIGNMENT);
     let mut filename_bytes = vec![0u8; (namesize + header_align).try_into().unwrap()];
     let filename_length: usize = (namesize - 1).try_into().unwrap();
     archive.read_exact(&mut filename_bytes)?;
@@ -280,6 +323,31 @@ fn read_filename<R: Read>(archive: &mut R, namesize: u32) -> Result<String> {
     Ok(filename.to_string())
 }
 
+/// Read only the file name from the next cpio object.
+///
+/// Read the next cpio object header, check the magic, skip the file data.
+/// Return the file name.
+pub fn read_filename_from_next_cpio_object<R: Read + SeekForward>(
+    archive: &mut R,
+) -> Result<String> {
+    let mut header = [0; CPIO_HEADER_LENGTH as usize];
+    archive.read_exact(&mut header)?;
+    check_begins_with_cpio_magic_header(&header)?;
+    let filesize = hex_str_to_u32(&header[54..62])?;
+    let namesize = hex_str_to_u32(&header[94..102])?;
+    let filename = read_filename(archive, namesize)?;
+    skip_file_content(archive, filesize)?;
+    Ok(filename)
+}
+
+fn skip_file_content<R: SeekForward>(archive: &mut R, filesize: u32) -> Result<()> {
+    if filesize == 0 {
+        return Ok(());
+    };
+    let skip = filesize + padding_needed_for(filesize.into(), CPIO_ALIGNMENT);
+    archive.seek_forward(skip.into())
+}
+
 #[cfg(test)]
 mod tests {
     use super::*;
@@ -360,6 +428,59 @@ mod tests {
         assert_eq!(size, 136);
     }
 
+    #[test]
+    fn test_header_write_filename_too_long() {
+        let filename = format!("this/path/is/way/t{}/long", "o".repeat(5000));
+        let header = Header::new(
+            42, 0o43_777, 1000, 2000, 1, 1720081471, 0, 37, 153, &filename,
+        );
+        let mut output = Vec::new();
+        let got = header.write(&mut output).unwrap_err();
+        assert_eq!(got.kind(), ErrorKind::InvalidData);
+        assert_eq!(
+            got.to_string(),
+            format!("Path '{filename}' exceeds filename length limit")
+        );
+    }
+
+    #[test]
+    fn test_header_write_with_alignment_exceeds_path_max() {
+        let path = "usr/lib/modules/6.16.0-13-generic/modules.dep";
+        let header = Header::new(42, 0o100_644, 0xAA, 0xBB, 1, 0x689CD1CC, 917184, 0, 0, path);
+        let mut output = Vec::new();
+        let size = header
+            .write_with_alignment(&mut output, Some(PATH_MAX as u32), 3956)
+            .unwrap();
+        assert_eq!(
+            std::str::from_utf8(&output).unwrap(),
+            "0707010000002A000081A4000000AA000000BB00000001689CD1CC\
+            000DFEC0000000000000000000000000000000000000002E00000000\
+            usr/lib/modules/6.16.0-13-generic/modules.dep\0",
+        );
+        assert_eq!(size, 156);
+    }
+
+    #[test]
+    fn test_header_write_with_alignment_near_path_max() {
+        let header = Header::new(
+            42, 0o100_644, 0xAA, 0xBB, 1, 0x689CD1CC, 917184, 0, 0, "data",
+        );
+        let mut output = Vec::new();
+        let size = header
+            .write_with_alignment(&mut output, Some(PATH_MAX as u32), 3988)
+            .unwrap();
+        assert_eq!(
+            std::str::from_utf8(&output).unwrap(),
+            format!(
+                "0707010000002A000081A4000000AA000000BB00000001689CD1CC\
+                000DFEC00000000000000000000000000000000000000FFE00000000\
+                data\0\0{}",
+                "\0".repeat(4088)
+            ),
+        );
+        assert_eq!(size, 4204);
+    }
+
     #[test]
     fn test_hex_str_to_u32() {
         let value = hex_str_to_u32(b"000003E8").unwrap();
@@ -397,4 +518,14 @@ mod tests {
         let header = Header::new(0, 0o100_644, 0, 0, 1, 1744150584, 0, 0, 0, ".");
         assert!(!header.is_root_directory());
     }
+
+    #[test]
+    fn test_padding_needed_for() {
+        assert_eq!(padding_needed_for(110, 4), 2);
+    }
+
+    #[test]
+    fn test_padding_needed_for_is_aligned() {
+        assert_eq!(padding_needed_for(32, 4), 0);
+    }
 }
