@@ -10,7 +10,7 @@ use glob::Pattern;
 
 use crate::compression::read_magic_header;
 use crate::filetype::*;
-use crate::header::Header;
+use crate::header::{read_filename_from_next_cpio_object, Header};
 use crate::libc::strftime_local;
 use crate::manifest::Manifest;
 use crate::ranges::Ranges;
@@ -107,26 +107,6 @@ fn format_time(timestamp: u32, now: i64) -> Result<String> {
 // TODO: Document hardlink structure
 type SeenFiles = HashMap<u128, String>;
 
-fn align_to_4_bytes(length: u32) -> u32 {
-    let unaligned = length % 4;
-    if unaligned == 0 {
-        0
-    } else {
-        4 - unaligned
-    }
-}
-
-/// Read only the file name from the next cpio object.
-///
-/// Read the next cpio object header, check the magic, skip the file data.
-/// Return the file name.
-fn read_filename_from_next_cpio_object<R: Read + SeekForward>(archive: &mut R) -> Result<String> {
-    let (filesize, filename) = Header::read_only_filesize_and_filename(archive)?;
-    let skip = filesize + align_to_4_bytes(filesize);
-    archive.seek_forward(skip.into())?;
-    Ok(filename)
-}
-
 fn read_cpio_and_print_filenames<R: Read + SeekForward, W: Write>(
     archive: &mut R,
     out: &mut W,
@@ -239,7 +219,7 @@ fn read_cpio_and_print_long_format<R: Read + SeekForward, W: Write>(
 }
 
 // Does the given file name matches one of the globbing patterns?
-pub fn filename_matches(filename: &str, patterns: &Vec<Pattern>) -> bool {
+fn filename_matches(filename: &str, patterns: &Vec<Pattern>) -> bool {
     for pattern in patterns {
         if pattern.matches(filename) {
             return true;
@@ -248,7 +228,7 @@ pub fn filename_matches(filename: &str, patterns: &Vec<Pattern>) -> bool {
     false
 }
 
-pub fn seek_to_cpio_end(archive: &mut File) -> Result<()> {
+fn seek_to_cpio_end(archive: &mut File) -> Result<()> {
     let cpio = CpioFilenameReader { archive };
     for f in cpio {
         f?;
@@ -299,7 +279,11 @@ pub fn print_cpio_archive_count<W: Write>(mut archive: File, out: &mut W) -> Res
 }
 
 // Return the size in bytes of the uncompressed data.
-pub fn create_cpio_archive(archive: Option<File>, log_level: u32) -> Result<u64> {
+pub fn create_cpio_archive(
+    archive: Option<File>,
+    alignment: Option<u32>,
+    log_level: u32,
+) -> Result<u64> {
     let source_date_epoch = get_source_date_epoch();
     let stdin = std::io::stdin();
     let buf_reader = std::io::BufReader::new(stdin);
@@ -310,7 +294,7 @@ pub fn create_cpio_archive(archive: Option<File>, log_level: u32) -> Result<u64>
     if log_level >= LOG_LEVEL_DEBUG {
         eprintln!("Writing cpio...");
     }
-    manifest.write_archive(archive, source_date_epoch, log_level)
+    manifest.write_archive(archive, alignment, source_date_epoch, log_level)
 }
 
 pub fn examine_cpio_content<W: Write>(mut archive: File, out: &mut W) -> Result<()> {
@@ -424,16 +408,6 @@ mod tests {
         }
     }
 
-    #[test]
-    fn test_align_to_4_bytes() {
-        assert_eq!(align_to_4_bytes(110), 2);
-    }
-
-    #[test]
-    fn test_align_to_4_bytes_is_aligned() {
-        assert_eq!(align_to_4_bytes(32), 0);
-    }
-
     #[test]
     fn test_get_cpio_archive_count_single() {
         let mut archive =
@@ -444,7 +418,7 @@ mod tests {
 
     #[test]
     fn test_list_cpio_content_compressed_parts() {
-        let archive = File::open(tests_path("lzop.cpio")).unwrap();
+        let archive = File::open(tests_path("gzip.cpio")).unwrap();
         let mut output = Vec::new();
         list_cpio_content(
             archive,
