@@ -11,7 +11,7 @@ use crate::extended_error::ExtendedError;
 use crate::filetype::*;
 use crate::header::Header;
 use crate::libc::{major, minor};
-use crate::{align_to_4_bytes, LOG_LEVEL_DEBUG, LOG_LEVEL_INFO};
+use crate::{LOG_LEVEL_DEBUG, LOG_LEVEL_INFO};
 
 #[derive(Debug, PartialEq)]
 struct Hardlink {
@@ -556,6 +556,7 @@ impl Archive {
     fn write<W: Write>(
         &self,
         output_file: &mut W,
+        alignment: Option<u32>,
         source_date_epoch: Option<u32>,
         mut size: u64,
         log_level: u32,
@@ -576,22 +577,19 @@ impl Archive {
             if log_level >= LOG_LEVEL_DEBUG {
                 writeln!(std::io::stderr(), "{header:?}")?;
             };
-            size += header.write(output_file)?;
+            size += header.write_with_alignment(output_file, alignment, size)?;
             match &file.filetype {
                 Filetype::Hardlink { key, index: _ } => {
                     if header.filesize > 0 {
                         let hardlink = self.hardlinks.get(key).unwrap();
-                        size += copy_file_with_padding(
-                            &hardlink.location,
-                            hardlink.filesize,
-                            output_file,
-                        )?;
+                        size += copy_file(&hardlink.location, hardlink.filesize, output_file)?;
+                        size += header.write_file_data_padding(output_file)?;
                     }
                 }
                 Filetype::Symlink { target } => {
                     output_file.write_all(target.as_bytes())?;
                     size += u64::try_from(target.len()).unwrap();
-                    size += write_padding(output_file, header.filesize)?;
+                    size += header.write_file_data_padding(output_file)?;
                 }
                 Filetype::EmptyFile
                 | Filetype::Directory
@@ -672,6 +670,7 @@ impl Manifest {
     pub fn write_archive(
         self,
         mut file: Option<std::fs::File>,
+        alignment: Option<u32>,
         source_date_epoch: Option<u32>,
         log_level: u32,
     ) -> Result<u64> {
@@ -683,16 +682,29 @@ impl Manifest {
             if archive.compression.is_uncompressed() {
                 if let Some(file) = file.as_mut() {
                     let mut writer = BufWriter::new(file);
-                    size = archive.write(&mut writer, source_date_epoch, size, log_level)?;
+                    size = archive.write(
+                        &mut writer,
+                        alignment,
+                        source_date_epoch,
+                        size,
+                        log_level,
+                    )?;
                     writer.flush()?;
                 } else {
                     let mut stdout = std::io::stdout().lock();
-                    size = archive.write(&mut stdout, source_date_epoch, size, log_level)?;
+                    size = archive.write(
+                        &mut stdout,
+                        alignment,
+                        source_date_epoch,
+                        size,
+                        log_level,
+                    )?;
+                    stdout.flush()?;
                 }
             } else {
                 let mut compressor = archive.compression.compress(file, source_date_epoch)?;
                 let mut writer = BufWriter::new(compressor.stdin.as_ref().unwrap());
-                size = archive.write(&mut writer, source_date_epoch, size, log_level)?;
+                size = archive.write(&mut writer, None, source_date_epoch, size, log_level)?;
                 writer.flush()?;
                 drop(writer);
                 let exit_status = compressor.wait()?;
@@ -710,40 +722,41 @@ impl Manifest {
     }
 }
 
-fn copy_file_with_padding<W: Write>(path: &str, filesize: u32, writer: &mut W) -> Result<u64> {
+fn copy_file<W: Write>(path: &str, filesize: u32, writer: &mut W) -> Result<u64> {
     let file = std::fs::File::open(path).map_err(|e| e.add_prefix(path))?;
     let mut reader = std::io::BufReader::new(file);
-    let mut copied_bytes = std::io::copy(&mut reader, writer)?;
+    let copied_bytes = std::io::copy(&mut reader, writer)?;
     if copied_bytes != filesize.into() {
         return Err(Error::new(
             ErrorKind::UnexpectedEof,
             format!("Copied {copied_bytes} bytes from {path} but expected {filesize} bytes."),
         ));
     }
-    copied_bytes += write_padding(writer, filesize)?;
     Ok(copied_bytes)
 }
 
-pub fn write_padding<W: Write>(file: &mut W, written_bytes: u32) -> Result<u64> {
-    let padding_len = align_to_4_bytes(written_bytes);
-    if padding_len == 0 {
-        return Ok(0);
-    }
-    let padding = vec![0u8; padding_len.try_into().unwrap()];
-    file.write_all(&padding)?;
-    Ok(padding_len.into())
-}
-
 #[cfg(test)]
 mod tests {
     use std::fs::{canonicalize, hard_link};
     use std::io::Read;
+    use std::path::Path;
 
     use super::*;
     use crate::temp_dir::TempDir;
     use crate::tests::TEST_LOCK;
     use crate::LOG_LEVEL_WARNING;
 
+    fn create_text_file_in_tmpdir<P: AsRef<Path>>(
+        tempdir: &TempDir,
+        filename: P,
+        content: &[u8],
+    ) -> String {
+        let path = tempdir.path.join(filename);
+        let mut text_file = std::fs::File::create(&path).unwrap();
+        text_file.write_all(content).unwrap();
+        path.into_os_string().into_string().unwrap()
+    }
+
     pub fn make_temp_dir_with_hardlinks() -> Result<TempDir> {
         let temp_dir = TempDir::new()?;
         let path = temp_dir.path.join("a");
@@ -1369,7 +1382,7 @@ mod tests {
         let manifest = Manifest::from_input(input.as_ref(), LOG_LEVEL_WARNING).unwrap();
         let file = std::fs::File::create(&path).unwrap();
         let size = manifest
-            .write_archive(Some(file), Some(1754439117), LOG_LEVEL_WARNING)
+            .write_archive(Some(file), None, Some(1754439117), LOG_LEVEL_WARNING)
             .unwrap();
         assert_eq!(size, 124);
         let mut written_file = std::fs::File::open(&path).unwrap();
@@ -1393,7 +1406,7 @@ mod tests {
         let manifest = Manifest::from_input(input.as_ref(), LOG_LEVEL_WARNING).unwrap();
         let file = std::fs::File::create(&path).unwrap();
         let size = manifest
-            .write_archive(Some(file), Some(1754439117), LOG_LEVEL_WARNING)
+            .write_archive(Some(file), None, Some(1754439117), LOG_LEVEL_WARNING)
             .unwrap();
         assert_eq!(size, 124);
         let mut written_file = std::fs::File::open(&path).unwrap();
@@ -1416,7 +1429,7 @@ mod tests {
         let manifest = Manifest::from_input(input.as_ref(), LOG_LEVEL_WARNING).unwrap();
         let file = std::fs::File::create(&path).unwrap();
         let size = manifest
-            .write_archive(Some(file), Some(1754439117), LOG_LEVEL_WARNING)
+            .write_archive(Some(file), None, Some(1754439117), LOG_LEVEL_WARNING)
             .unwrap();
         assert_eq!(size, 124);
         let mut written_file = std::fs::File::open(&path).unwrap();
@@ -1439,7 +1452,7 @@ mod tests {
         let manifest = Manifest::from_input(input.as_ref(), LOG_LEVEL_WARNING).unwrap();
         let file = std::fs::File::create(&path).unwrap();
         let size = manifest
-            .write_archive(Some(file), Some(1754439117), LOG_LEVEL_WARNING)
+            .write_archive(Some(file), None, Some(1754439117), LOG_LEVEL_WARNING)
             .unwrap();
         assert_eq!(size, 124);
         let mut written_file = std::fs::File::open(&path).unwrap();
@@ -1461,9 +1474,14 @@ mod tests {
         let input = b"#cpio: lzop -9\n";
         let manifest = Manifest::from_input(input.as_ref(), LOG_LEVEL_WARNING).unwrap();
         let file = std::fs::File::create(&path).unwrap();
-        let size = manifest
-            .write_archive(Some(file), Some(1754439117), LOG_LEVEL_WARNING)
-            .unwrap();
+        let got = manifest.write_archive(Some(file), None, Some(1754439117), LOG_LEVEL_WARNING);
+        if got
+            .as_ref()
+            .is_err_and(|e| e.to_string() == "Program 'lzop' not found in PATH.")
+        {
+            return;
+        }
+        let size = got.unwrap();
         assert_eq!(size, 124);
         let mut written_file = std::fs::File::open(&path).unwrap();
         let mut output = Vec::new();
@@ -1494,7 +1512,7 @@ mod tests {
         let manifest = Manifest::from_input(input.as_ref(), LOG_LEVEL_WARNING).unwrap();
         let file = std::fs::File::create(&path).unwrap();
         let size = manifest
-            .write_archive(Some(file), Some(1754439117), LOG_LEVEL_WARNING)
+            .write_archive(Some(file), None, Some(1754439117), LOG_LEVEL_WARNING)
             .unwrap();
         assert_eq!(size, 124);
         let mut written_file = std::fs::File::open(&path).unwrap();
@@ -1519,7 +1537,7 @@ mod tests {
         let manifest = Manifest::from_input(input.as_ref(), LOG_LEVEL_WARNING).unwrap();
         let file = std::fs::File::create(&path).unwrap();
         let size = manifest
-            .write_archive(Some(file), Some(1754439117), LOG_LEVEL_WARNING)
+            .write_archive(Some(file), None, Some(1754439117), LOG_LEVEL_WARNING)
             .unwrap();
         assert_eq!(size, 124);
         let mut written_file = std::fs::File::open(&path).unwrap();
@@ -1542,7 +1560,7 @@ mod tests {
         let manifest = Manifest::new(vec![archive], 0o022);
         let file = std::fs::File::create(temp_dir.path.join("initrd.img")).unwrap();
         let got = manifest
-            .write_archive(Some(file), None, LOG_LEVEL_WARNING)
+            .write_archive(Some(file), None, None, LOG_LEVEL_WARNING)
             .unwrap_err();
         assert!(
             matches!(got.kind(), ErrorKind::Other if got.to_string() == "false failed: exit status: 1")
@@ -1578,9 +1596,9 @@ mod tests {
             ),
             File::new(
                 Filetype::Symlink {
-                    target: "usr/bin".into(),
+                    target: "usr/sbin".into(),
                 },
-                "bin",
+                "sbin",
                 0o777,
                 0x336,
                 0x45,
@@ -1588,11 +1606,11 @@ mod tests {
             ),
             File::new(Filetype::Fifo, "initctl", 0o600, 0x337, 0x46, 0x6862B88A),
             File::new(Filetype::Socket, "notify", 0o777, 0x338, 0x47, 0x681DE2C2),
-            File::new(Filetype::EmptyFile, "fstab", 0x644, 0x339, 0x48, 0x6E44C280),
+            File::new(Filetype::EmptyFile, "fstab", 0o644, 0x339, 0x48, 0x6E44C280),
         ]);
         let mut output = Vec::new();
         let size = archive
-            .write(&mut output, Some(0x6B49D200), 0, LOG_LEVEL_WARNING)
+            .write(&mut output, None, Some(0x6B49D200), 0, LOG_LEVEL_WARNING)
             .unwrap();
         assert_eq!(
             std::str::from_utf8(&output).unwrap(),
@@ -1606,15 +1624,15 @@ mod tests {
             00000000000000000000000000002E0E00008C750000000800000000\
             console\0\0\0\
             070701000000030000A1FF00000336000000450000000162373894\
-            00000007000000000000000000000000000000000000000400000000\
-            bin\0\0\0usr/bin\0\
+            00000008000000000000000000000000000000000000000500000000\
+            sbin\0\0usr/sbin\
             07070100000004000011800000033700000046000000016862B88A\
             00000000000000000000000000000000000000000000000800000000\
             initctl\0\0\0\
             070701000000050000C1FF000003380000004700000001681DE2C2\
             00000000000000000000000000000000000000000000000700000000\
             notify\0\0\0\0\
-            07070100000006000086440000033900000048000000016B49D200\
+            07070100000006000081A40000033900000048000000016B49D200\
             00000000000000000000000000000000000000000000000600000000\
             fstab\0\
             070701000000000000000000000000000000000000000100000000\
@@ -1624,6 +1642,71 @@ mod tests {
         assert_eq!(size, 952);
     }
 
+    #[test]
+    fn test_archive_write_aligned() {
+        let tempdir = TempDir::new().unwrap();
+        let example = create_text_file_in_tmpdir(
+            &tempdir,
+            "example.txt",
+            b"This is just an example text file!\n",
+        );
+        let small = create_text_file_in_tmpdir(&tempdir, "small.txt", b"shorter than alignment\n");
+        let mut hardlinks = HashMap::new();
+        hardlinks.insert(42, Hardlink::with_references(example, 35, 1));
+        hardlinks.insert(99, Hardlink::with_references(small, 23, 1));
+        let archive = Archive::with_files_and_hardlinks(
+            vec![
+                File::new(Filetype::Directory, ".", 0o755, 0x333, 0x42, 0x6841897B),
+                File::new(
+                    Filetype::Hardlink { key: 42, index: 1 },
+                    "example",
+                    0o644,
+                    0x339,
+                    0x48,
+                    0x6E44C280,
+                ),
+                File::new(
+                    Filetype::Hardlink { key: 99, index: 1 },
+                    "small.txt",
+                    0o644,
+                    0x339,
+                    0x48,
+                    0x6E44C280,
+                ),
+            ],
+            hardlinks,
+        );
+        let mut output = Vec::new();
+        let size = archive
+            .write(
+                &mut output,
+                Some(32),
+                Some(0x6B49D200),
+                0,
+                LOG_LEVEL_WARNING,
+            )
+            .unwrap();
+        assert_eq!(
+            std::str::from_utf8(&output).unwrap(),
+            "07070100000000000041ED0000033300000042000000026841897B\
+            00000000000000000000000000000000000000000000000200000000\
+            .\0\
+            07070100000001000081A40000033900000048000000016B49D200\
+            00000023000000000000000000000000000000000000002200000000\
+            example\0\0\0\
+            \0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\
+            This is just an example text file!\n\0\
+            07070100000002000081A40000033900000048000000016B49D200\
+            00000017000000000000000000000000000000000000000A00000000\
+            small.txt\0\
+            shorter than alignment\n\0\
+            070701000000000000000000000000000000000000000100000000\
+            00000000000000000000000000000000000000000000000B00000000\
+            TRAILER!!!\0\0\0\0",
+        );
+        assert_eq!(size, 560);
+    }
+
     #[test]
     fn test_archive_write_hardlinks() {
         let temp_dir = make_temp_dir_with_hardlinks().unwrap();
@@ -1658,7 +1741,7 @@ mod tests {
         );
         let mut output = Vec::new();
         let size = archive
-            .write(&mut output, None, 0, LOG_LEVEL_WARNING)
+            .write(&mut output, None, None, 0, LOG_LEVEL_WARNING)
             .unwrap();
         assert_eq!(
             std::str::from_utf8(&output).unwrap(),
