@@ -4,7 +4,7 @@
 use std::env;
 use std::error::Error;
 use std::fs::File;
-use std::io::{Read, Write};
+use std::io::{ErrorKind, Read, Write};
 use std::process::{Command, Output, Stdio};
 use std::time::SystemTime;
 
@@ -26,6 +26,12 @@ fn get_command() -> Command {
     Command::new(program)
 }
 
+fn program_not_available(program: &str) -> bool {
+    let mut cmd = Command::new(program);
+    cmd.arg("--help");
+    cmd.output().is_err_and(|e| e.kind() == ErrorKind::NotFound)
+}
+
 trait ExitCodeAssertion {
     fn assert_failure(self, expected_code: i32) -> Self;
     fn assert_success(self) -> Self;
@@ -200,6 +206,94 @@ fn test_create_cpio_file() -> Result<(), Box<dyn Error>> {
     Ok(())
 }
 
+#[test]
+fn test_create_data_align() -> Result<(), Box<dyn Error>> {
+    let temp_dir = TempDir::new()?;
+    let path = temp_dir.create("example.txt", b"This is just an example text file!\n")?;
+    let mut cmd = get_command();
+    cmd.args(["--create", "--data-align", "16"]);
+    let process = cmd.stdin(Stdio::piped()).stdout(Stdio::piped()).spawn()?;
+    let mut stdin = process.stdin.as_ref().unwrap();
+    let manifest =
+        format!("/usr\t\t\t\t\t\t1681992796\n{path}\tusr/file\t\t644\t3\t7\t1755046204\n");
+    stdin.write_all(manifest.as_bytes())?;
+    let output = process.wait_with_output()?;
+    assert_eq!(
+        std::str::from_utf8(&output.stdout).unwrap(),
+        "07070100000000000041ED00000000000000000000000264412C5C\
+        00000000000000000000000000000000000000000000000400000000\
+        usr\0\0\0\
+        07070100000001000081A4000000030000000700000001689BE13C\
+        00000023000000000000000000000000000000000000000E00000000\
+        usr/file\0\0\
+        \0\0\0\0\
+        This is just an example text file!\n\0\
+        070701000000000000000000000000000000000000000100000000\
+        00000000000000000000000000000000000000000000000B00000000\
+        TRAILER!!!\0\0\0\0",
+    );
+    Ok(())
+}
+
+#[test]
+fn test_create_data_align_negative_number() -> Result<(), Box<dyn Error>> {
+    let mut cmd = get_command();
+    cmd.args(["--create", "--data-align", "-42", "/tmp/initrd"]);
+
+    cmd.output()?
+        .assert_failure(2)
+        .assert_stderr_contains("Error: --data-align must be a positive number")
+        .assert_stdout("");
+    Ok(())
+}
+
+#[test]
+fn test_create_data_align_not_a_multiple_of_four() -> Result<(), Box<dyn Error>> {
+    let mut cmd = get_command();
+    cmd.args(["--create", "--data-align", "7", "/tmp/initrd"]);
+
+    cmd.output()?
+        .assert_failure(2)
+        .assert_stderr_contains("Error: --data-align must be a multiple of 4 bytes")
+        .assert_stdout("");
+    Ok(())
+}
+
+#[test]
+fn test_create_data_align_zero() -> Result<(), Box<dyn Error>> {
+    let mut cmd = get_command();
+    cmd.args(["--create", "--data-align", "0", "/tmp/initrd"]);
+
+    cmd.output()?
+        .assert_failure(2)
+        .assert_stderr_contains("Error: --data-align must be a multiple of 4 bytes")
+        .assert_stdout("");
+    Ok(())
+}
+
+#[test]
+fn test_create_uncompressed_plus_zstd_on_stdout() -> Result<(), Box<dyn Error>> {
+    let mut cmd = get_command();
+    cmd.arg("--create");
+    let process = cmd.stdin(Stdio::piped()).stdout(Stdio::piped()).spawn()?;
+    let mut stdin = process.stdin.as_ref().unwrap();
+    stdin.write_all(b"#cpio\n/usr\t\t\t\t\t\t1681992796\n#cpio: zstd -2\n")?;
+    let output = process.wait_with_output()?;
+    assert_eq!(
+        output.stdout,
+        b"07070100000000000041ED00000000000000000000000264412C5C\
+        00000000000000000000000000000000000000000000000400000000\
+        usr\0\0\0\
+        070701000000000000000000000000000000000000000100000000\
+        00000000000000000000000000000000000000000000000B00000000\
+        TRAILER!!!\0\0\0\0\
+        (\xb5/\xfd\x04P\x15\x01\0\xc8070701\
+        010B0TRAILER!!!\0\
+        \0\0\0\x03\x10\0\x19\xde\x89?F\x95\xfb\x16m",
+    );
+    Ok(())
+}
+
 #[test]
 fn test_count_cpio_archives() -> Result<(), Box<dyn Error>> {
     let mut cmd = get_command();
@@ -224,6 +318,9 @@ fn test_count_unexpected_argument() -> Result<(), Box<dyn Error>> {
 #[test]
 fn test_examine_compressed_cpio() -> Result<(), Box<dyn Error>> {
     for compression in ["bzip2", "gzip", "lz4", "lzop", "xz", "zstd"] {
+        if program_not_available(compression) {
+            continue;
+        }
         let mut cmd = get_command();
         cmd.arg("-e").arg(format!("tests/{compression}.cpio"));
 
@@ -356,6 +453,9 @@ fn test_invalid_pattern() -> Result<(), Box<dyn Error>> {
 #[test]
 fn test_list_content_compressed_cpio() -> Result<(), Box<dyn Error>> {
     for compression in ["bzip2", "gzip", "lz4", "lzma", "lzop", "xz", "zstd"] {
+        if program_not_available(compression) {
+            continue;
+        }
         let mut cmd = get_command();
         cmd.arg("-t").arg(format!("tests/{compression}.cpio"));
 
@@ -367,6 +467,32 @@ fn test_list_content_compressed_cpio() -> Result<(), Box<dyn Error>> {
     Ok(())
 }
 
+#[test]
+fn test_list_content_compressed_cpio_verbose() -> Result<(), Box<dyn Error>> {
+    for compression in ["bzip2", "gzip", "lz4", "lzma", "lzop", "xz", "zstd"] {
+        if program_not_available(compression) {
+            continue;
+        }
+        let mut cmd = get_command();
+        cmd.arg("-tv").arg(format!("tests/{compression}.cpio"));
+        cmd.env("TZ", "UTC");
+
+        cmd.output()?
+            .assert_stderr("")
+            .assert_success()
+            .assert_stdout(
+                "drwxrwxr-x   2 root     root            0 Apr 14  2024 .\n\
+                 drwxrwxr-x   2 root     root            0 Apr 14  2024 path\n\
+                 -rw-rw-r--   1 root     root            8 Apr 14  2024 path/file\n\
+                 drwxrwxr-x   2 root     root            0 Apr 14  2024 .\n\
+                 drwxrwxr-x   2 root     root            0 Apr 14  2024 usr\n\
+                 drwxrwxr-x   2 root     root            0 Apr 14  2024 usr/bin\n\
+                 -rw-rw-r--   1 root     root           56 Apr 14  2024 usr/bin/sh\n",
+            );
+    }
+    Ok(())
+}
+
 #[test]
 fn test_list_content_parts_compressed_cpio() -> Result<(), Box<dyn Error>> {
     let mut cmd = get_command();
@@ -391,6 +517,23 @@ fn test_list_content_single_cpio() -> Result<(), Box<dyn Error>> {
     Ok(())
 }
 
+#[test]
+fn test_list_content_single_cpio_verbose() -> Result<(), Box<dyn Error>> {
+    let mut cmd = get_command();
+    cmd.arg("-tv").arg("tests/single.cpio");
+    cmd.env("TZ", "UTC");
+
+    cmd.output()?
+        .assert_stderr("")
+        .assert_success()
+        .assert_stdout(
+            "drwxrwxr-x   2 root     root            0 Apr 14  2024 .\n\
+             drwxrwxr-x   2 root     root            0 Apr 14  2024 path\n\
+             -rw-rw-r--   1 root     root            8 Apr 14  2024 path/file\n",
+        );
+    Ok(())
+}
+
 #[test]
 fn test_list_content_single_cpio_with_pattern() -> Result<(), Box<dyn Error>> {
     let mut cmd = get_command();
